/**
* The contents of this file are subject to the Mozilla Public 
* License Version 1.1 (the "License"); you may not use this 
* file except in compliance with the License. You may obtain a
* copy of the License at http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an 
* "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express 
* or implied. See the License for the specific language 
* governing rights and limitations under the License.
*
* The Original Code is this file as it was released on
* December 24, 2004.
*
* The Initial Developer of the Original Code is Terry Yuen. 
* Portions created by Terry Yuen are Copyright (C) 2004.
* All Rights Reserved.
**/

var settings = {
  prefs: null,
  tabCount: 1,
  tabCountChanged: false,
  colourChanged: false,
  siteIconsChanged: false,

  startup : function() {
    try {
      settings.prefs = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefService).getBranch(null);
      settings.loadSettings();
      settings.prefillColours();
    } catch(e) {
      alert("The following error has occurred:\n" + e);
    }
  },

  loadSettings : function() {
    settings.tabCount = settings.prefs.getIntPref("hashcolouredtabs.tab_count_trigger");
    var lt0 = settings.prefs.getIntPref("hashcolouredtabs.light_base");
    var lti = settings.prefs.getIntPref("hashcolouredtabs.light_inc");
    var ltn = settings.prefs.getIntPref("hashcolouredtabs.light_count");
    var showIcons = (settings.prefs.getBoolPref("browser.chrome.site_icons")
        || settings.prefs.getBoolPref("browser.chrome.favicons"));

    document.getElementById("tab_count").value = settings.tabCount;
    document.getElementById("favicons").checked = !showIcons;

    if(lti == 30 && (lt0 - 42) % lti == 0 && ltn > 0 && ltn <= 6) {
      var min = (lt0 - 42) / lti;
      var max = min + ltn - 1;
      for(var i=0; i<6; i++) {
        document.getElementById("c" + i).checked = (i >= min && i <= max);
      }
    }
  },

  checkTabCount : function() {
    settings.tabCountChanged = true;
    var str = document.getElementById("tab_count").value;

    var num = parseInt(str);
    if(isNaN(num)) {
      alert("= \"" + str + "\"?! \n= {0,1,2,3,...,10}");
      return false;
    }
    else if(num < -1 || num > 500) {
      alert("-1 < \"" + num + "\" < 500");
      return false;
    }
    settings.tabCount = num;
    return true;
  },

  prefillColours : function() {
    const hueTable = new Array(
    0xff0000, 0xff6600, 0xffcc00, 0xccff00, 0x66ff00, 
    0x00ff00, 0x00ff66, 0x00ffcc, 0x00ccff, 0x0066ff, 
    0x0000ff, 0x6600ff, 0xcc00ff, 0xff00cc, 0xff0066
    );
    var cw = document.getElementById("colourWheel");
    var tmp,r,b,g;
    for(var i=0; i<6; i++) {
      row = cw.childNodes[i];
      for(var j=1; j<16; j++) {
        tmp = hueTable[j - 1];
        r = (tmp & 0xff0000) >> 16;
        g = (tmp & 0x00ff00) >> 8;
        b = (tmp & 0x0000ff) >> 0;

        tmp = 42 + 30*i - 128;
        if(tmp < 0) {
          tmp /= -128.0;
          r = Math.min(Math.round(r + tmp*(255 - r)), 255);
          g = Math.min(Math.round(g + tmp*(255 - g)), 255);
          b = Math.min(Math.round(b + tmp*(255 - b)), 255);
        }
        else if(tmp > 0) {
          tmp = (128 - tmp) / 128.0;
          r = Math.max(Math.round(tmp*r), 0);
          g = Math.max(Math.round(tmp*g), 0);
          b = Math.max(Math.round(tmp*b), 0);
        }

        row.childNodes[j].style.backgroundColor = "rgb("+r+","+g+","+b+")";
      }
    }
  },

  changeColour : function(row) {
    settings.colourChanged = true;
    var tmp = false;
    var first = 0;
    for(var i=0; i<6; i++) {
      tmp = document.getElementById("c" + i).checked;
      if((tmp && row != i) || (!tmp && row == i)) {
        first = i;
        break;
      }
    }

    var last = 0;
    for(var i=5; i>=0; i--) {
      tmp = document.getElementById("c" + i).checked;
      if((tmp && row != i) || (!tmp && row == i)) {
        last = i;
        break;
      }
    }

    if(first < row && row < last) {
      if(row - first < last - row)
        first = row;
      else
        last = row;
    }

    if(last > first) {
      for(var i=0; i<6; i++) {
        if(i != row)document.getElementById("c" + i).checked = (i >= first && i <= last);
      }
    }
  },

  changeSiteIcon : function() {
    settings.siteIconChanged = true;
  },

  acceptDialog : function() {
    if(settings.tabCountChanged) {
      settings.prefs.setIntPref("hashcolouredtabs.tab_count_trigger", settings.tabCount);
    }
    if(settings.colourChanged) {
      var first = 0;
      for(var i=0; i<6; i++) {
        if(document.getElementById("c" + i).checked) {
          first = i;
          break;
        }
      }
      var last = 5;
      for(var i=5; i>=0; i--) {
        if(document.getElementById("c" + i).checked) {
          last = i;
          break;
        }
      }
      settings.prefs.setIntPref("hashcolouredtabs.light_base", (42 + first*30));
      settings.prefs.setIntPref("hashcolouredtabs.light_count", ((last - first) + 1));

      var node = (typeof window.opener == "object") ? window.opener : null;
      while(node != null) {
        if(typeof node.hashColouredTabsCode == "object") {
          node.hashColouredTabsCode.loadSettings();
          break;
        }
        node = (typeof node.opener == "object") ? node.opener : null;
      }
      if(node == null)alert("Restart Firefox to see changes.");
    }
    if(settings.siteIconChanged) {
      if(document.getElementById("favicons").checked) {
        settings.prefs.setBoolPref("browser.chrome.site_icons", false);
        settings.prefs.setBoolPref("browser.chrome.favicons", false);
      } else {
        settings.prefs.setBoolPref("browser.chrome.site_icons", true);
        settings.prefs.setBoolPref("browser.chrome.favicons", true);
      }
    }
  }
};